/******************************** CPPFile *****************************

* FileName [CEGen.cpp]

* PackageName [main]

* Synopsis [Method definitions of CEGen class.]

* SeeAlso [CEGen.h]

* Author [Sagar Chaki]

* Copyright [ Copyright (c) 2002 by Carnegie Mellon University. All
* Rights Reserved. This software is for educational purposes only.
* Permission is given to academic institutions to use, copy, and
* modify this software and its documentation provided that this
* introductory message is not removed, that this software and its
* documentation is used for the institutions' internal research and
* educational purposes, and that no monies are exchanged. No guarantee
* is expressed or implied by the distribution of this code. Send
* bug-reports and/or questions to: chaki+@cs.cmu.edu. ]

**********************************************************************/

#include <cassert>
#include <string>
#include <list>
#include <set>
#include <vector>
#include <map>
#include <bitset>
using namespace std;

#include "Node.h"
#include "Action.h"
#include "Database.h"
#include "Predicate.h"
#include "PredSet.h"
#include "ContLoc.h"
#include "LocalAbsLts.h"
#include "GlobalCEDag.h"
#include "ConcCEDag.h"
#include "ImplState.h"
#include "ProcManager.h"
#include "CEDagVerifier.h"
#include "CEGen.h"

/*********************************************************************/
//do all the stuff
/*********************************************************************/
void CEGen::Run()
{
  GlobalCEDag::Initialize();
  GlobalCEDag::Compute();    

  //if checking simulation of ERROR state reachability
  if((Database::CONF_TYPE == Database::CONF_SIMUL) || (Database::CONF_TYPE == Database::CONF_REACH)) {
    RunFsp();
  }
  //if checking LTL formula
  else if(Database::CONF_TYPE == Database::CONF_LTL) {
    RunLtl();
  }
  //impossible
  else assert(false);
}

/*********************************************************************/
//the top-level routine when we are checking for simulation or ERROR
//state reachability
/*********************************************************************/
void CEGen::RunFsp()
{
  //generate CE for each component
  for(size_t i = 0;i < Database::procManagers.size();i++) {
    ProcManager &procManager = Database::procManagers[i];
    ConcCEDag concDag;
    assert(concDag.ComputeLocLabelFsp(procManager));
    concDag.Display(procManager);
  }
}

/*********************************************************************/
//the top-level routine when we are checking for LTL property
/*********************************************************************/
void CEGen::RunLtl()
{
  //generate CE for each component
  for(size_t i = 0;i < Database::procManagers.size();i++) {
    ProcManager &procManager = Database::procManagers[i];
    ConcCEDag concDag;
    assert(concDag.ComputeLocLabelLtl(procManager));
    concDag.Display(procManager);
  }
}

/*********************************************************************/
//end of CEGen.cpp
/*********************************************************************/
