/**************************** CPPHeaderFile ***************************

* FileName [ConcCEDag.h]

* PackageName [main]

* Synopsis [Header file for ConcCEDag class.]

* Description [This class encapsulates a concrete counterexample dag.]

* SeeAlso [ConcCEDag.cpp]

* Author [Sagar Chaki]

* Copyright [ Copyright (c) 2002 by Carnegie Mellon University.  All
* Rights Reserved. This software is for educational purposes only.
* Permission is given to academic institutions to use, copy, and
* modify this software and its documentation provided that this
* introductory message is not removed, that this software and its
* documentation is used for the institutions' internal research and
* educational purposes, and that no monies are exchanged. No guarantee
* is expressed or implied by the distribution of this code. Send
* bug-reports and/or questions to: chaki+@cs.cmu.edu. ]

**********************************************************************/

#ifndef __CONCCEDAG_H__
#define __CONCCEDAG_H__

//other classes needed
class ProcManager;
class ImplState;
class ImStSet;

/*********************************************************************/
//a concrete counterexample dag
/*********************************************************************/
class ConcCEDag
{
 private:
  //label from states to control locations
  vector<const ContLoc*> locLabel;

  //the labeling of nodes with actions
  vector<Action> actLabel;

  //the transition relation
  vector< set<int> > trans;

  void ComputeTopoWithRoot(const int node,set<int> &visited,vector<int> &topo) const;
  void ComputeLocLabelNoLtsAbs(const ProcManager &procManager); 
  void DisplayWithRoot(const int node,list<const ContLoc*> &locStack,list<Action> &actStack,const ProcManager &procManager) const;
  void DisplayPath(const list<const ContLoc*> &locStack,const list<Action> &actStack,const ProcManager &procManager) const;

 public:
  ConcCEDag() {}
  ConcCEDag(const ConcCEDag &rhs) { *this = rhs; }

  const ConcCEDag &operator = (const ConcCEDag &rhs);
  bool operator == (const ConcCEDag &rhs) const;
  bool operator < (const ConcCEDag &rhs) const;

  void ComputeTopo(vector<int> &topo) const;
  bool ComputeLocLabelFsp(const ProcManager &procManager);
  bool ComputeLocLabelLtl(ProcManager &procManager);
  void DagToPaths(set<ConcCEDag> &concPaths,const ProcManager &procManager) const;
  void Display(const ProcManager &procManager) const;

  friend class CEDagVerifier;
  friend class PredAbsRefiner;
};

#endif //__CONCCEDAG_H__

/*********************************************************************/
//end of ConcCEDag.h
/*********************************************************************/
