/**************************** CPPHeaderFile ***************************

* FileName [GlobalAbsLts.h]

* PackageName [main]

* Synopsis [Header file for GlobalAbsLts class.]

* Description [This class encapsulates the global abstract LTS
* i.e. the parallel composition of all the local abstract LTSs.]

* SeeAlso [GlobalAbsLts.cpp]

* Author [Sagar Chaki]

* Copyright [ Copyright (c) 2002 by Carnegie Mellon University.  All
* Rights Reserved. This software is for educational purposes only.
* Permission is given to academic institutions to use, copy, and
* modify this software and its documentation provided that this
* introductory message is not removed, that this software and its
* documentation is used for the institutions' internal research and
* educational purposes, and that no monies are exchanged. No guarantee
* is expressed or implied by the distribution of this code. Send
* bug-reports and/or questions to: chaki+@cs.cmu.edu. ]

**********************************************************************/

#ifndef __GLOBALABSLTS_H__
#define __GLOBALABSLTS_H__

class GlobalAbsLts
{
 private:
  static void CartesianProduct(const vector< set<int> > &arg,size_t index,int factor,int curr,set<int> &res);

 public:
  //the set of reachable states
  static set<int> reach;

  GlobalAbsLts() { assert(false); }
  ~GlobalAbsLts() {}

  static void GetInitStates(set<int> &res);
  static void GetPrevInitStates(set<int> &res);
  static void GetSuccsOnAction(int comp,const Action &act,set<int> &res);
  static void GetPredsOnAction(int comp,const Action &act,set<int> &res);
  static void GetActions(set<Action> &res);
  static int GetStateNum();
  static int GetPrevStateNum();
  static void GetSuccsChanged(set<int> &res);
  static void GetPrevSuccsOnAction(int comp,const Action &act,set<int> &res);
  static set<Expr> GetPropositions(int comp);

  //split a composite global state into states for individual
  //components of the program
  static vector<int> CompToAbsStates(int global);
};

#endif //__GLOBALABSLTS_H__

/*********************************************************************/
//end of GlobalAbsLts.h
/*********************************************************************/
