/**************************** CPPHeaderFile ***************************

* FileName [InputProcessor.h]

* PackageName [main]

* Synopsis [Header file for the InputProcessor class. This class is
* responsible for processing the input, getting user options and file
* names, parsing the files, constructing the control flow graph and
* creating the necessary predicates at each control location.]

* Description []

* SeeAlso [InputProcessor.cpp]

* Author [Sagar Chaki]

* Copyright [ Copyright (c) 2002 by Carnegie Mellon University.  All
* Rights Reserved. This software is for educational purposes only.
* Permission is given to academic institutions to use, copy, and
* modify this software and its documentation provided that this
* introductory message is not removed, that this software and its
* documentation is used for the institutions' internal research and
* educational purposes, and that no monies are exchanged. No guarantee
* is expressed or implied by the distribution of this code. Send
* bug-reports and/or questions to: chaki+@cs.cmu.edu. ]

**********************************************************************/

#ifndef __INPUTPROCESSOR_H__
#define __INPUTPROCESSOR_H__

class InputProcessor
{
 private:
  static Expr CreateMainPred(ProcInfo &mainInfo,const Expr &mainPred);
  static void RemoveSkipLocations();
  static ContLoc *GetClosestNonSkipLoc(ContLoc *arg);
  static void RemoveUselessBranches();
  static void DoLiveVarAnalysis();
  static void BuildInitialMaps();
  static void ComputeNecessaryPredicates();
 public:
  static bool Run();
};

#endif //__INPUTPROCESSOR_H__

/*********************************************************************/
//end of InputProcessor.h
/*********************************************************************/
