/**************************** CPPHeaderFile ***************************

* FileName [LocalAbsLts.h]

* PackageName [main]

* Synopsis [Header file for LocalAbsLts class.]

* Description [This class encapsulates a local abstract LTS.]

* SeeAlso [LocalAbsLts.cpp]

* Author [Sagar Chaki]

* Copyright [ Copyright (c) 2002 by Carnegie Mellon University.  All
* Rights Reserved. This software is for educational purposes only.
* Permission is given to academic institutions to use, copy, and
* modify this software and its documentation provided that this
* introductory message is not removed, that this software and its
* documentation is used for the institutions' internal research and
* educational purposes, and that no monies are exchanged. No guarantee
* is expressed or implied by the distribution of this code. Send
* bug-reports and/or questions to: chaki+@cs.cmu.edu. ]

**********************************************************************/

#ifndef __LOCALABSLTS_H__
#define __LOCALABSLTS_H__

//other classes needed
class ProcManager;

class LocalAbsLts
{
 private:
  //the id of the proc manager corresponding to this lts
  size_t pmid;

  //the total number of abstract states in this iteration and the
  //previous iteration
  size_t stateNum,prevStateNum;
  
  //initial abstract states in this iteration and the previous iteration.
  set<int> init,prevInit;

  //the map from states to propositions
  map< int,set<Expr> > propMap;

  //abstract actions
  set<Action> acts;

  //the set of states whose successors might have changed due to
  //action guided refinement
  set<int> succsChanged;

  //the abstract successor and predecessor transition relation
  map< int,map< Action,set<int> > > succMap;
  map< int,map< Action,set<int> > > predMap;

  //the abstract successor relation from the previous iteration
  map< int,map< Action,set<int> > > prevSuccMap;
  

 public:
  LocalAbsLts() {}
  LocalAbsLts(size_t p) { pmid = p; stateNum = 0; prevStateNum = 0; }
  LocalAbsLts(const LocalAbsLts &rhs) { *this = rhs; }

  const LocalAbsLts &operator = (const LocalAbsLts &rhs);

  size_t GetStateNum() const { return stateNum; }
  size_t GetPrevStateNum() const { return prevStateNum; }
  const set<int> &GetInit() const { return init; }
  const set<int> &GetPrevInit() const { return prevInit; }
  const set<Action> &GetActs() const { return acts; }
  const set<int> &GetSuccsChanged() const { return succsChanged; }
  const set<Expr> GetPropositions(int state) const;

  set<int> GetSuccsOnAction(int from,const Action &act);
  set<int> GetPredsOnAction(int from,const Action &act) const;
  set<int> GetPrevSuccsOnAction(int from,const Action &act);
  void Display();

  friend class ProcManager;
  friend class LtsAbsRefiner;
};

#endif //__LOCALABSLTS_H__

/*********************************************************************/
//end of LocalAbsLts.h
/*********************************************************************/
