/**************************** CPPHeaderFile ***************************

* FileName [PredSet.h]

* PackageName [main]

* Synopsis [Header file for PredSet class.]

* Description [This class encapsulates a set of predicates.]

* SeeAlso [PredSet.cpp]

* Author [Sagar Chaki]

* Copyright [ Copyright (c) 2002 by Carnegie Mellon University.  All
* Rights Reserved. This software is for educational purposes only.
* Permission is given to academic institutions to use, copy, and
* modify this software and its documentation provided that this
* introductory message is not removed, that this software and its
* documentation is used for the institutions' internal research and
* educational purposes, and that no monies are exchanged. No guarantee
* is expressed or implied by the distribution of this code. Send
* bug-reports and/or questions to: chaki+@cs.cmu.edu. ]

**********************************************************************/

#ifndef __PREDSET_H__
#define __PREDSET_H__

//other classes needed
class Predicate;

//two types of a predicate valuation - the first one is used for
//storage and the second for easier manipulation
typedef bitset<Database::PVBITS> PVal;
typedef pair< PVal,vector<short> > PredVal;

class PredSet
{
 private:
  //the vector of predicates
  vector<Predicate> preds;
  
  //the number of valuations
  PVal valNum;

  //the factors for the valuations of various predicates
  vector<PVal> factors;

  //the cache used to store relationships between predicates
  static map< pair< pair<const PredSet*,const PredSet*>,pair< const list<Expr>,const list<Expr> > >,map< pair<size_t,short>,pair<size_t,short> > > cache;
  
  void ComputeValNumAndFactors();
  pair< set<Expr>,set<Expr> > ToExprSets(const PredVal &val) const;
  void CollectPossibleValuations(vector<PVal> &ret,
				 const vector< vector <short> > &vals,
				 const PVal &sum,size_t pos) const;
  void ClearThisCache();
  void CreateCache(const PredSet &before,const list<Expr> &lhsList,const list<Expr> &rhsList,
		   map< pair<size_t,short>,pair<size_t,short> > &entry) const;

 public:
  PredSet();
  PredSet(const PredSet &rhs) { *this = rhs; }
  const PredSet &operator = (const PredSet &rhs);

  inline static unsigned long PValToUL(const PVal &arg) { return arg.to_ulong(); }
  inline static PVal ULToPVal(unsigned long arg) { return PVal(arg); }
  inline static void ClearCache() { cache.clear(); }

  string ToString() const;
  bool AddPred(const Predicate &p);
  const vector<Predicate> &GetPreds() const { return preds; }
  PredVal GetInitPredValuation() const;
  bool IsValidPredValuation(const PredVal &val) const;
  PredVal GetNextPredValuation(const PredVal &val) const;
  set<Expr> ToExpr(const PredVal &val) const;
  vector<PVal> GetPossibleValuations(const PredSet &before,const PVal &bsval,
				     const list<Expr> &lhsList,
				     const list<Expr> &rhsList) const;
  PredVal PValToPredVal(const PVal &arg) const;
  void Merge(const PredSet &rhs);
  void Clear();
  bool IsEmpty() const;
};

#endif //__PREDSET_H__

/*********************************************************************/
//end of PredSet.h
/*********************************************************************/
